/*
 * Copyright (c) 2020, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "ti_msp_dl_config.h"

#define TIMER_CAPTURE_PERIOD (8000)

volatile bool gCaptureCompleted = false;
/*
 * CC0 is used for width capture
 * CC1 is used for period capture
 */
volatile int TimerG_CaptureResult_CC0 = 0;
volatile int TimerG_CaptureResult_CC1[2] = {0};
/*
 * TimerCaptureRisingEdgeFirst means it will detect the first rise edge and start capture.
 * else, it will detect the first fall edge and start capture.
 */
#define TimerCaptureRisingEdgeFirst

volatile uint32_t pwmPeriod;
volatile uint32_t pwmDuty;

/* Used to observe a set of captured data */
volatile uint32_t pwmPeriod_array[100] = {0};
__attribute__((unused)) volatile uint32_t pwmDuty_array[100] = {0};
volatile uint32_t CaptureCnts = 0;

int main(void)
{
    SYSCFG_DL_init();

    /*
     * Workaround 1: CC register will load a wrong value when set CCOND and LCOND the same edge action,
     * set LCOND as 0, then do a capture calculation in the interrupt routine.
     * Following shows the configuration of combined capture mode with rising or falling edge comes firstly.
     * This will update in the following SDK version.
     */
    #ifdef TimerCaptureRisingEdgeFirst
    DL_Timer_setCaptureCompareCtl(CAPTURE_0_INST, DL_TIMER_CC_MODE_CAPTURE,
        (DL_TIMER_CC_LCOND_NONE | DL_TIMER_CC_CCOND_TRIG_FALL), DL_TIMER_CC_0_INDEX);

    DL_Timer_setCaptureCompareCtl(CAPTURE_0_INST, DL_TIMER_CC_MODE_CAPTURE,
        (DL_TIMER_CC_LCOND_NONE | DL_TIMER_CC_CCOND_TRIG_RISE), DL_TIMER_CC_1_INDEX);
    #else
    DL_Timer_setCaptureCompareCtl(CAPTURE_0_INST, DL_TIMER_CC_MODE_CAPTURE,
        (DL_TIMER_CC_LCOND_NONE | DL_TIMER_CC_CCOND_TRIG_RISE), DL_TIMER_CC_0_INDEX);

    DL_Timer_setCaptureCompareCtl(CAPTURE_0_INST, DL_TIMER_CC_MODE_CAPTURE,
        (DL_TIMER_CC_LCOND_NONE | DL_TIMER_CC_CCOND_TRIG_FALL), DL_TIMER_CC_1_INDEX);
    #endif

    NVIC_EnableIRQ(CAPTURE_0_INST_INT_IRQN);

    DL_TimerG_startCounter(CAPTURE_0_INST);

    DL_TimerG_startCounter(PWM_0_INST);

    while (1) {
        __WFE();
    }
}

void CAPTURE_0_INST_IRQHandler(void)
{
    switch (DL_TimerG_getPendingInterrupt(CAPTURE_0_INST)) {
        case DL_TIMERG_IIDX_CC1_DN:

            /* Update the captured value for CC0 - Pulse Width */
            TimerG_CaptureResult_CC0 = DL_Timer_getCaptureCompareValue(
                    CAPTURE_0_INST, DL_TIMER_CC_0_INDEX);

            /* Update the captured value for CC1 - Period */
            TimerG_CaptureResult_CC1[0] = TimerG_CaptureResult_CC1[1];
            TimerG_CaptureResult_CC1[1] = DL_Timer_getCaptureCompareValue(
                    CAPTURE_0_INST, DL_TIMER_CC_1_INDEX);

            gCaptureCompleted = true;
            break;
        default:
            break;
    }

    /*
     * Process the value after both Period and Duty captured.
     */
    if (true == gCaptureCompleted){

        gCaptureCompleted = false;
        /*
         * If use period mode, do NOT set LCOND to load counter automatically.
         * A rising edge is detected and capture compare register 1 loads the counter
         * value in the interrupt routine.
         * Therefore the period of the pwm signal will be the previous capture compare
         * register value minus the newer capture compare register value (down counting).
         */
        #ifdef TimerCaptureRisingEdgeFirst

        /* Be careful whether it will underflow or overflow when data processing */
        if(TimerG_CaptureResult_CC1[1] > TimerG_CaptureResult_CC1[0]){
            pwmPeriod = TimerG_CaptureResult_CC1[0] - TimerG_CaptureResult_CC1[1] + TIMER_CAPTURE_PERIOD;
        } else {
            pwmPeriod = TimerG_CaptureResult_CC1[0] - TimerG_CaptureResult_CC1[1];
        }
        /*
         * In Capture Combined Mode the number of timer clocks that the signal
         * remained high is captured in capture compare register 0.
         * In  order to determine timer clocks between a rising edge and the next
         * falling edge. It is necessary to subtract the capture compare register 1
         * value (rising edge time point) from capture compare register 0 value
         * (falling edge time point). In down counting, refers to:
         *    t_high = TIMER_X_CC01[Channel 1] - TIMER_X_CC01[Channel 0]
         * To calculate the duty cycle use the following formula:
         *  pwmDuty = (t_high / pwmPeriod) * 100
         */
        if(TimerG_CaptureResult_CC0 > TimerG_CaptureResult_CC1[0]){
            TimerG_CaptureResult_CC0 -= TIMER_CAPTURE_PERIOD;
        }   /* Be careful whether it will underflow or overflow when data processing */

        pwmDuty = ((TimerG_CaptureResult_CC1[0] - TimerG_CaptureResult_CC0)* 100) / pwmPeriod;
        #else

        if(TimerG_CaptureResult_CC1[1] > TimerG_CaptureResult_CC1[0]){
            TimerG_CaptureResult_CC1[0] += TIMER_CAPTURE_PERIOD;
        }   /* Be careful whether it will underflow or overflow when data processing */
        if(TimerG_CaptureResult_CC0 < TimerG_CaptureResult_CC1[1]){
            TimerG_CaptureResult_CC0 += TIMER_CAPTURE_PERIOD;
        }   /* Be careful whether it will underflow or overflow when data processing */

        pwmPeriod = TimerG_CaptureResult_CC1[0] - TimerG_CaptureResult_CC1[1];
        pwmDuty = ((-TimerG_CaptureResult_CC1[1] + TimerG_CaptureResult_CC0)* 100) / pwmPeriod;
        #endif

        /*
         * Set a capture value array for observation
         */
        pwmPeriod_array[CaptureCnts] = pwmPeriod;
        pwmDuty_array[CaptureCnts] = pwmDuty;
        CaptureCnts++;
        if (CaptureCnts >= 100){
            CaptureCnts =0;
            __BKPT(0);
        }
    }

}
